/*
 * Пример работы с библиотекой I2C
 * Почти готовые функции:
 * - ScanDevice
 * - isReady
 */
#include "STM32.h"
#include "IO_Digital.h"
#include "I2C.h"
#include "UART.h"
#include "string.h"
#include "stdint.h"
#include "Dump.h"
#include "AT24Cxx.h"

#define   ADDRESS_EEPROM 0x50

I2C         myI2C(I2C2, false);                             // Класс I2C
uart        myUART(USART1, false);
AT24Cxx     EEPROM(&myI2C, 0x50, AT24C64, Size_Add_16bit);  // Пользуемся ранее соданным классом

struct sSaveType              // Объявляем структуру
{
    uint32_t  Data;
    uint8_t   Serial;
    float     Fl;
} ;

int main(void)
{
  int16_t Temp;
  uint8_t Counts = 0;
  uint8_t Buff[256] = {0,};
  struct sSaveType SaveType;  // Теперича у неё есть имя

  SystemClock_Config(Quartz_8);
  myUART.init(256000);
  myI2C.Init(Mode_Slow);

  uint8_t BuffFF[256] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                         0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                         0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                         0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                         0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                         0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                         0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,};
//
  uint8_t BuffFill[256] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,
                        17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
                        33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48,
                        49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64,
                        65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80,
                        0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

  char msg[128] = { 0, };

  /*
   * Сканируем количество и адреса устройств и выводим
   */
  Counts = myI2C.ScanDevice(Buff);
  if(Counts)
  {
    for(uint8_t I=0; I<Counts; I++)
    {
      sprintf(msg, "Device: %d Address - 0x%02X OK \n\r", I, Buff[I]);
      myUART.print(msg);
    }
  }

  // Печатаем буфер вывода, что бы убедится что он чист
  myUART.println("\n\r----------------- Buffer cleared ----------------------");
  Dump_EEPROM(0, 32, 3, Buff);                    // Выводим буфер что бы убедится что он очищен

   //Забиваем 128 байт EEPROM для чистоты эксперимента
  EEPROM.Write(0x0000, BuffFF, 8*16);

  // Читаем и выводим 128 байт для контроля
  myUART.println("-------------------- Control read FF ----------------------");
  Temp = EEPROM.Read(0, Buff, 128);
  sprintf(msg, "\n\rAddress - 0x%02X Error - %d \n\r\n\r", ADDRESS_EEPROM, Temp);
  myUART.print(msg);
  delay(20);
  Dump_EEPROM(0, 32, 4, Buff);

  // Пишем и читаем 32 байта идущими по порядку
  EEPROM.Write(0, BuffFill, 32);
  myUART.println("------------------- Test 32 byte --------------------------");
  Temp = EEPROM.Read(0, Buff, 128);
  sprintf(msg, "\n\rAddress - 0x%02X Error - %d \n\r\n\r", ADDRESS_EEPROM, Temp);
  myUART.print(msg);
  delay(20);
  Dump_EEPROM(0, 32, 4, Buff);

  // Пишем и читаем один байт по определённому адресу
  EEPROM.Write(0x0073, 0x11);                                                   // Пробуем писать байт 0x5F по адресу 0x0073
  myUART.println("--------- Test byte 0x11, Address 0x73 --------------------");
  Temp = EEPROM.Read(0, Buff, 128);
  sprintf(msg, "\n\rAddress - 0x%02X Error - %d \n\r\n\r", ADDRESS_EEPROM, Temp);
  myUART.print(msg);
  delay(20);
  Dump_EEPROM(0, 32, 4, Buff);

  // Инициализируем структуру и пишем её с 20го адреса
  SaveType.Data   = 123456;
  SaveType.Serial = 0xAA;
  SaveType.Fl     = 1246.345;
  EEPROM.Write(0x20, &SaveType, sizeof(SaveType));  // Пишем структуру с адреса 0х20
  myUART.println("--------- Address 0x20. Saved structure ------------------");
  Temp = EEPROM.Read(0, Buff, 128);
  sprintf(msg, "\n\rAddress - 0x%02X Error - %d \n\r\n\r", ADDRESS_EEPROM, Temp);
  myUART.print(msg);
  delay(20);
  Dump_EEPROM(0, 32, 4, Buff);

  // Затираем структуру для контроля
  SaveType.Data   = 0;
  SaveType.Serial = 0;
  SaveType.Fl     = 0;
  myUART.println("------------ Print sructure clear -------------------------");
  myUART.print("Data   : "); myUART.println(SaveType.Data);
  myUART.print("Serial : 0x"); myUART.println(SaveType.Serial);
  myUART.print("Fl     : "); myUART.println(SaveType.Fl);

  EEPROM.Read(0x20, &SaveType, sizeof(SaveType));
  myUART.println("------------ Print readed sructure ------------------------");
  myUART.print("Data   : "); myUART.println(SaveType.Data);
  myUART.print("Serial : "); myUART.println(SaveType.Serial, HEX);
  myUART.print("Fl     : "); myUART.println(SaveType.Fl, 4);


  while(1)
  {
    delay(10);
  }
}


/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

